/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                    CHIPCON CC1010 EXAMPLE PROGRAM            *
 *      ***   + +   ***                TWO PLAYER TENNIS GAME                *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * This is the world's first tennis game using frequency hopping.            *
 * It demonstrates the use of the FHP library.                               *
 *                                                                           *
 * INSTRUCTIONS:                                                             *
 * Use two evaluation boards. Set up one as a master by turning the ADC to a *
 * high value and the other as a slave by turning it to a low value. The     *
 * game starts immediately.                                                  *
 *                                                                           *
 * The ball can come in any of four positions, corresponding to the four     *
 * LEDs and buttons. A LED lights up to indicate where the ball comes. The   *
 * player has to press the corresponding button to hit the ball. The player  *
 * directs the ball at any of the four positions of the opponent by turning  *
 * the ADC. There is a time limit on the strokes that is reduced gradually.  *
 * If the player misses the ball (presses the wrong button) or is to slow    *
 * (does not press a button before the timeout), she loses.                  *
 *                                                                           *
 * When the game is over, all four LEDs light up. Then the green LED of the  *
 * the player that won goes on and the red LED of the player that lost.      *
 * Thereafter all LEDs goes off and there is a break of 1 second before the  *
 * next game.                                                                *
 *                                                                           *
 * HOW IT WORKS:                                                             *
 * The two most significant bits of the ADC reading is sent to the opponent. *
 * The opponent lights the corresponding LED and polls the buttons. If the   *
 * correct button is pressed before the timeout, an ADC reading is performed *
 * and sent to the other player. Otherwise, it notifies the other player     *
 * that it lost and displays game over and the result on the LEDs.           *
 *                                                                           *
 * The timing is performed by a simple counter that is decremented every     *
 * time the buttons are polled. The starting value of the counter is reduced *
 * by 5 per cent every time the ball is hit, but is reset on game over.      *
 *****************************************************************************
 * Author:              OGR                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 *                                                                           *
 * $Log: tennis.c,v $
 * Revision 1.1  2003/07/29 11:20:49  tos
 * Initial version in CVS.
 *
 *                                                                           *
 ****************************************************************************/

#include <chipcon/hal.h>
#include <chipcon/cul.h>
#include <chipcon/cc1010eb.h>

#define CLK_FREQ 14746

#define initial (CLK_FREQ*2)
word cnt_start = initial;

// The receiving function simply sets a flag.
// The whole processing should not be performed here
// since this function is called from withing the
// interrupt service routine. That would block the FHP.
bool newMessage = FALSE;
void notify() {
	newMessage = TRUE;
}

// Waits 'time' milliseconds
void wait(word time) {
	while (time>=255) {
		time -= 255;
		halWait(255, CLK_FREQ);
	}
	halWait(time, CLK_FREQ);
}

// Reads the two MSB of the ADC
byte getDirection() {
    ADC_SAMPLE_SINGLE();
	return ADC_GET_SAMPLE_8BIT()>>6;
}

xdata byte buffer[1];

// Reads and sends the ADC
void sendADC() {
	buffer[0] = getDirection();
	newSending(buffer,1,1);
}

// Flash all the LEDs on game over
void LED_blink() {
	RLED = GLED = YLED = BLED = LED_ON;
	wait(500);
	RLED = GLED = YLED = BLED = LED_OFF;
}

// This function controls the game
void processMessage() {
	word cnt = cnt_start;
	byte pressed = 255;
	cnt_start *= 0.95;
	RLED = GLED = YLED = BLED = LED_OFF;
	if (message[0]>=4) {
		// The opponent missed or timed out, I won
		cnt_start = initial;
		LED_blink();
		GLED = LED_ON;
		wait(1000);
		GLED = LED_OFF;
	} else {
		// Show where the ball comes
		switch (message[0]) {
			case 0: RLED = LED_ON; break;
			case 1: YLED = LED_ON; break;
			case 2: GLED = LED_ON; break;
			case 3: BLED = LED_ON; break;
		}
		// Poll the buttons
		while (--cnt) {
			if (SW1_PRESSED) { pressed = 0; break; }
			if (SW2_PRESSED) { pressed = 1; break; }
			if (SW3_PRESSED) { pressed = 2; break; }
			if (SW4_PRESSED) { pressed = 3; break; }
		}
		if (pressed == message[0]) {
			// Correct button pressed within the time limit
			RLED = GLED = YLED = BLED = LED_OFF;
			sendADC();
		} else {
			// Timeout or wrong button, I lost
			buffer[0] = 4;
			newSending(buffer,1,1);
			cnt_start = initial;
			LED_blink();
			RLED = LED_ON;
			wait(1000);
			RLED = LED_OFF;
			wait(1000);
			sendADC();
		}
	}
}

// Main function performs the necessary initializations.
int main(void){
	bool ism;
	
	//Turn off watchdog
    //(otherwise the CC1010 will reset after a predefined time)
    WDT_ENABLE(FALSE);

    // Set optimum settings for speed and low power consumption
    MEM_NO_WAIT_STATES();
    FLASH_SET_POWER_MODE(FLASH_STANDBY_BETWEEN_READS);

	X32_INPUT_SOURCE(X32_USING_CRYSTAL);
	X32_ENABLE(TRUE);

	//All I/Os are inputs after reset. Make I/Os connected to LEDs outputs
    RLED=YLED=GLED=BLED=LED_OFF;
    RLED_OE(TRUE); YLED_OE(TRUE); GLED_OE(TRUE); BLED_OE(TRUE);

    // Setup ADC, select the CC1010EB pot meter, and turn it on
    halConfigADC(ADC_MODE_SINGLE | ADC_REFERENCE_VDD, CLK_FREQ, 0);
    ADC_SELECT_INPUT(ADC_INPUT_AD0);
    ADC_POWER(TRUE); // Power up ADC from sleep mode

	ism = getDirection()>=2;
	initializeFHP(ism);
	setOwnID(1);
	setReceivingFunction(notify);
	setMode(ACTIVE);

	if (!ism) sendADC();

	// Poll the newMessage flag
    while(1) {
		while (!newMessage);
		newMessage = FALSE;
		processMessage();
   }
}

