/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                    CHIPCON CC1010 EXAMPLE PROGRAM            *
 *      ***   + +   ***         FREQUENCY HOPPING TEMPERATURE SENSOR         *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * This program demonstrates the use of the FHP library and the power saving *
 * modes of the chip.                                                        *
 *                                                                           *
 * INSTRUCTIONS:                                                             *
 * Use two or more evaluation boards. Connect one of them to a terminal      *
 * window via serial port 0 (38400). Let this one be a master by turning the *
 * potmeter on the evaluation board to a low value before start-up. Place    *
 * the other evaluation boards within radio range and let them be slaves by  *
 * turning the potmeter to a high value before start-up.                     *
 * The slaves will then measure the temperature at their location and send   *
 * the information to the master, which will print it to the terminal        *
 * window.                                                                   *
 *                                                                           *
 * HOW IT WORKS:                                                             *
 * The master and the slaves share the same code. The boards choose between  *
 * being master and slave by reading the value of the ADC, which is          *
 * controled by the potmeter.
 * The slaves first send a request to the master for a new ID, and it        *
 * returns an unused value. At regular intervals the slaves wake up from     *
 * idle mode, switch to the main crystal oscillator, read the temperature,   *
 * transmit it to the master, go back to the 32 kHz crystal oscillator, and  *
 * enter idle mode again.                                                    *
 * If the first acquisition attempt fails, the slaves simply go back to      *
 * power down mode until it is time to send a new temperature reading.       *
 *****************************************************************************
 * Author:              OGR                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 *                                                                           *
 * $Log: temperature.c,v $
 * Revision 1.1  2003/07/29 11:18:37  tos
 * Initial version in CVS.
 *
 *                                                                           *
 ****************************************************************************/

#include <chipcon/hal.h>
#include <chipcon/cul.h>
#include <chipcon/cc1010eb.h>
#include <stdio.h>

#define CLK_FREQ 14746

#define GET_ID 1
#define TEMPERATURE 2

#define MASTER 127
#define NEWADDRESS 126

byte ID = NEWADDRESS;
byte newID = 1;
bool messageReceived = FALSE;

xdata byte buffer[10];

// This function is called when the master receives
// a message from one of the slaves. It could either
// be a request for an ID or a temperature reading,
// and the corresponding actions are taken.
void masterProcessMessage() {
	ulong milliseconds;
	word minutes, seconds;
	word adc;
	float temp;
	byte i;
	if ((messageLength==1) && (message[0]==GET_ID)) {
		buffer[0] = newID;
		newSending(buffer, 1, NEWADDRESS);
		printf("A new sensor has joined us! It is given the number %u.\n",(word)newID);
		++newID;
	} else if ((messageLength==4) && (message[0]==TEMPERATURE)) {
		milliseconds = getTime()/1000;
		minutes = milliseconds/60000;
		milliseconds -= minutes*60000;
		seconds = milliseconds/1000;
		milliseconds -= seconds*1000;
		adc = (message[2]<<8) | message[3];
		temp = adc - 492;
		temp /= 8.192;
		printf("At %02u:%02u.%03u sensor %u says the temperature is %3.2f\n",minutes,seconds,(word)milliseconds,(word)message[1],temp);
		} else {
			printf("Unreckognized message:");
			for (i=0; i<messageLength; ++i) {
				printf(" %u",(word)message[i]);
			}
			printf("\n");
		}
}

// This function is called when a slave receives
// a message from the master, which is always a
// new ID.
void slaveProcessMessage() {
	setMode(PASSIVE);
	ID = message[0];
	setOwnID(ID);
	printf("new ID: %u\n",(word)ID);
	messageReceived = FALSE;
	setReceivingFunction(NULL);
}

// The slave sends a message every 10th second.
// Before a new ID is obtained, it is a request for
// an ID. Thereafter, temperature readings are sent.
void sendTemperature() interrupt INUM_RTC {
	if (ID == NEWADDRESS) {
		setMode(ACTIVE);
		buffer[0] = GET_ID;
		newSending(buffer,1,MASTER);
		printf("Request for ID sent\n");
	} else {
		XOSC_ENABLE(TRUE);
		halWait(5,32);
		useXOSC();
		ADC_SAMPLE_SINGLE();
		buffer[0] = TEMPERATURE;
		buffer[1] = ID;
		buffer[2] = ADDATH;
		buffer[3] = ADDATL;
		newSending(buffer,4,MASTER);
		printf("Sending temperature\n");
	}
	INT_SETFLAG(INUM_RTC, INT_CLR); //	RTCIF = 0;
}

// If acquisition failed, cancel the sending
// and wait until next time the RTC ISR is called.
void continueAfterWait() {
	printf("Synchronization failed!\n");
	cancelSending();
}

//Main function
int main(void){
	
	//Turn off watchdog
    //(otherwise the CC1010 will reset after a predefined time)
    WDT_ENABLE(FALSE);

    // Set optimum settings for speed and low power consumption
    MEM_NO_WAIT_STATES();
    FLASH_SET_POWER_MODE(FLASH_STANDBY_BETWEEN_READS);

	X32_INPUT_SOURCE(X32_USING_CRYSTAL);
	X32_ENABLE(TRUE);

	//All I/Os are inputs after reset. Make I/Os connected to LEDs outputs
    RLED=YLED=GLED=BLED=LED_OFF;
    RLED_OE(TRUE); YLED_OE(TRUE); GLED_OE(TRUE); BLED_OE(TRUE);

 	//Setup UART0 with polled I/O
    UART0_SETUP(38400, CC1010EB_CLKFREQ, UART_NO_PARITY | UART_RX_TX | UART_POLLED);

    // Setup ADC, select the CC1010EB pot meter, and turn it on
    halConfigADC(ADC_MODE_SINGLE | ADC_REFERENCE_VDD, CLK_FREQ, 0); // 0: Threshold value (not used in this program)
    ADC_SELECT_INPUT(ADC_INPUT_AD0);
    ADC_POWER(TRUE); // Power up ADC from sleep mode
    ADC_SAMPLE_SINGLE();

	if (ADC_GET_SAMPLE_8BIT() > 50) {
		printf("Starting up as slave...\n");
	    halConfigADC(ADC_MODE_SINGLE | ADC_REFERENCE_INTERNAL_1_25, CC1010EB_CLKFREQ, 0);
	    ADC_SELECT_INPUT(ADC_INPUT_AD1);
	    ADC_POWER(TRUE);
		halConfigRealTimeClock(10);
		setOwnID(NEWADDRESS);
		initializeFHP(FALSE);
		setReceivingFunction(slaveProcessMessage);
		setSynchronizationFailedFunction(continueAfterWait);
		setMode(ACTIVE);
	    INT_PRIORITY(INUM_RTC, INT_HIGH);
	    INT_ENABLE(INUM_RTC, INT_ON);
	    RTC_RUN(TRUE);
		while (1) {
			if (getSendStatus()==IDLE && ID!=NEWADDRESS) {
				useX32();
				XOSC_ENABLE(FALSE);
				ENTER_IDLE_MODE();
			}
		}
	} else {
		printf("Starting up as master...\n");
	    ADC_POWER(FALSE); // Power down the ADC
		initializeFHP(TRUE);
		setOwnID(MASTER);
		setMode(ACTIVE);
		RTstart();
		setReceivingFunction(masterProcessMessage);
	}

    //Infinite loop
    while(1);
}

